/* Nessus
 * Copyright (C) 1998 - 2004 Renaud Deraison
 *
 * Adapted for the Netbios Auditing Tool by Javier Fernandez-Sanguino
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Signals handlers
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "sighandler.h"
#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif


void (*set_signal(int signum, void (*handler)(int)))(int)
{
	struct sigaction saNew,saOld;

	/* Init new handler */
	sigfillset(&saNew.sa_mask);
	sigdelset(&saNew.sa_mask, SIGALRM); /* make sleep() work */

	saNew.sa_flags = 0;
# ifdef HAVE_SIGNAL_SA_RESTORER
	saNew.sa_restorer = 0; /* not avail on Solaris - jordan */
# endif
	saNew.sa_handler = handler;

	sigaction(signum, &saNew, &saOld);
	return saOld.sa_handler;
}


void sighandler(sign)
	int sign;
{
	char * sig = NULL;
	int murderer = 0;

	switch(sign)
	{
		case SIGTERM:
			sig = "TERM";
			close_sockets();
			break;
		case SIGINT :
			sig = "INT";
			close_sockets();
			break;
		case SIGPIPE :
			sig = "PIPE";
			close_sockets();
			break;
		case SIGSEGV :
#ifdef HAVE__EXIT
			signal(SIGSEGV, _exit);
#else
			signal(SIGSEGV, exit);
#endif
			sig = "SEGV";
			break;
		default:
			sig = "unknown signal";
	}
	fprintf(stderr, "Received signal %s!\n", sig);
#ifdef HAVE__EXIT
	_exit(1);
#else
	exit(1);
#endif

}


void sighand_segv()
{
#ifdef HAVE__EXIT
	signal(SIGSEGV, _exit);
#else
	signal(SIGSEGV, exit);
#endif
	fprintf(stderr, "Received SIGSEGV!\n");
#ifdef HAVE__EXIT
	_exit(1);
#else
	exit(1);
#endif
}

